## Load some library
library("e1071")
library(foreach)
library(doParallel)

registerDoParallel(4) # Set the number of cores (usually, number of available core +1

load("astrostat.RData")

n=nrow(x)
d=ncol(x)
C = max(y)

numberTrain = 100 # Select "numberTrain" per class for training
numberTest = 6300-numberTrain # The remaining is for validation

## Initialization of the training/validation sets
xt = matrix(0,numberTrain*C,d)
yt = matrix(0,numberTrain*C,1)
xT = matrix(0,numberTest*C,d)
yT = matrix(0,numberTest*C,1)

for (i in 1:C)
{
    t = which(y==i)
    set.seed(i)
    ts = sample(t) # Permute ramdomly the samples for class i
    xt[(1+numberTrain*(i-1)):(numberTrain*i),]=x[ts[1:numberTrain],]
    yt[(1+numberTrain*(i-1)):(numberTrain*i),]=y[ts[1:numberTrain],]
    
    xT[(1+numberTest*(i-1)):(numberTest*i),]=x[ts[(numberTrain+1):6300],]
    yT[(1+numberTest*(i-1)):(numberTest*i),]=y[ts[(numberTrain+1):6300],]       
}

## Set the CV range of search (dependent to your data)
gamma = 2^(-8:0)
cost=10^(-2:6)
CVT = matrix(0,nrow=length(cost),ncol=length(gamma))

## Serial version
TIME = Sys.time()
for (i in 1:length(cost))
{
    for (j in 1:length(gamma))
        {
            set.seed(0)
            model = svm(xt,yt,cost=cost[i],gamma=gamma[j],type="C",cross=5,cachesize=512)
            CVT[i,j]=model$tot.accuracy
        }
}
print(Sys.time()-TIME)
print(CVT)

## Get the position of the maximum
indices=which(CVT == max(CVT), arr.ind = TRUE)

## Learn the model
model = svm(xt,yt,cost=cost[indices[1,1]],gamma=gamma[indices[1,2]],type="C",cachesize=512)

## Predict the validation samples
yp = predict(model,xT)

## Confusion matrix
confu = table(yT,yp)
OA = sum(diag(confu))/sum(confu)


## ## ## Parallel Version
## TIME = Sys.time()
## CV = foreach(i = 1:length(cost),.combine = rbind) %dopar% {
##     temp = matrix(0,nrow=length(gamma),1)
##     for (j in 1:length(gamma)){
##         set.seed(0)
##         model = svm(xt,yt,cost=cost[i],gamma=gamma[j],type="C",cross=5,cachesize=512)
##         temp[j] = model$tot.accuracy
##     }
##     return(c(i,temp))
## }
## print(Sys.time()-TIME)
## print(CV)
