# Load SVM package
library(e1071)

# Generate the data
n=100
set.seed(0)
rho = 1.5+rnorm(n/2,0,0.2)
theta = 2*pi*runif(n/2)
mylevels = seq(-0,0,5)

X = matrix(0,nrow=n,ncol=2)
X[1:(n/2),1]=rnorm(n/2,0,0.25)
X[1:(n/2),2]=rnorm(n/2,0,0.25)

X[(n/2+1):n,1]=rho*cos(theta)
X[(n/2+1):n,2]=rho*sin(theta)
Y = c(rep(-1,n/2),rep(1,n/2))
# Generate the grid
xx = seq(-2,2,length=100)
yy = seq(-2,2,length=100)
XT = as.matrix(expand.grid(xx,yy))

## Learn the model with a given set of parameters
model = svm(X,Y,cost=10,gamma=0.5,type="C-classification")
## Predict on the grid samples
pred = predict(model,XT,decision.values = TRUE)
df = matrix(attr(pred, "decision.values"),ncol=100,nrow=100)
contour(xx,yy,df,levels=mylevels,xaxs='i',yaxs='i')
points(X[Y==-1,1],X[Y==-1,2],pch=1,xlim = c(-2,2), ylim = c(-2,2),xlab = "", ylab = "")
points(X[Y==1,1],X[Y==1,2],pch=2)